import sys
import gzip
import re
import pdb
from operator import itemgetter


def gzopen( filename ):
    return gzip.open(filename) if filename.endswith(".gz") else open(filename)

def read_file( infile ):

    rows = [ tuple(line.strip().split('\t'))
             for line in infile ]

    # remove header
    if rows[0][0] == '#': header = rows.pop(0)

    cols = zip(*rows)
    cols[0] = tuple(map(int, cols[0]))
    cols = tuple(cols)
    rows = tuple(zip(*cols))

    return cols

def segment_gold_phrases_type_level(gold, lexicon):
    predictions = []
    r = re.compile("^-+$")
    for phrase in gold:
        predicted_phrase = []
        for segmented_word in phrase.split():
            #word = segmented_word.replace('-','') if segmented_word != "-" else segmented_word
            word = segmented_word.replace('-','') if not r.match(segmented_word) else segmented_word
            assert word
            predicted_segmentation = "-".join(lexicon[word])
            predicted_phrase.append( predicted_segmentation )
        predictions.append( "\t".join(predicted_phrase) )
    return tuple(predictions)

def get_split_points(sw):
    segs = sw.split('-')
    x = len(segs[0])
    res = []
    for i in range(1,len(segs)):
        res.append( x )
        x += len(segs[i])
    return tuple(res)

def score(gold, prediction):
    assert len(gold) == len(prediction)
    # NOTE: counts not used
    correct = 0
    total_guess = 0
    total_actual = 0
    for gphrase,pphrase in zip(gold,prediction):
        gwords = gphrase.split()
        pwords = pphrase.split()
        assert len(gwords) == len(pwords)
        for gsw,psw in zip(gwords,pwords):
            gw = gsw.replace('-','')
            pw = psw.replace('-','')
            assert gw == pw
            gsplits = set( get_split_points(gsw) )
            psplits = set( get_split_points(psw) )
            correct += len( gsplits & psplits )
            total_guess += len( psplits )
            total_actual += len( gsplits )
    recall = 1.0 * correct / total_actual if total_actual > 0 else 0.0
    precision = 1.0 * correct / total_guess if total_guess > 0 else 0.0
    rp = recall + precision
    f1 = 2.0 * recall * precision / rp if rp > 0 else 0.0
    return [recall,precision,f1]
            
def split_data(col):
    test_set = [ v for i,v in enumerate(col) if i % 5 == 0 ]
    train_set = [ v for i,v in enumerate(col) if i % 5 != 0 ]
    return (test_set,train_set)

def write_as_seg_pos(counts, phrases, outfile):
    assert len(counts) == len(phrases)
    d = {}
    for count,phrase in zip(counts,phrases):
        words = phrase.split()
        nw = len(words)
        for i in range(count):
            print >>outfile, "# %d" % nw
            for w in words:
                if w not in d: d[w] = len(d) + 1
                a = d[w]
                print >>outfile, "+ %d 1" % a, # dummy lookup and input words
                segs = w.split('-')
                ns = len(segs)
                print >>outfile, ns,
                print >>outfile, " ".join(segs),
                tags = ['NOUN']*ns
                print >>outfile, " ".join(tags)
    print >>outfile,"== lookup lexicon"
    for k,v in sorted(d.iteritems(),key=itemgetter(1)):
        print >>outfile, k
    print >>outfile,"== input lexicon"
    print >>outfile,"*dummy*"

def write_as_sb08(counts, phrases, outfile):
    assert len(counts) == len(phrases)
    for count,phrase in zip(counts,phrases):
        print >>outfile, "%d\t%s" % (count,phrase)
    
def main():
    filename = sys.argv[1]
    outstem = sys.argv[2]
    cols = read_file( gzopen(filename) )

    write_as_seg_pos(cols[0], cols[1], open(outstem+".hebrew.all.seg_pos",'w'))
    write_as_seg_pos(cols[0], cols[4], open(outstem+".arabic.all.seg_pos",'w'))

    write_as_sb08( cols[0], cols[1], open(outstem+".hebrew.all.fpg",'w'))
    write_as_sb08( cols[0], cols[4], open(outstem+".arabic.all.fpg",'w'))

    (test_counts, train_counts) = split_data(cols[0])
    (test_hebrew, train_hebrew) = split_data(cols[1])
    (test_arabic, train_arabic) = split_data(cols[4])

    write_as_sb08( test_counts, test_hebrew, open(outstem+".hebrew.test.fpg",'w'))
    write_as_sb08( test_counts, test_arabic, open(outstem+".arabic.test.fpg",'w'))
    write_as_sb08( train_counts, train_hebrew, open(outstem+".hebrew.train.fpg",'w'))
    write_as_sb08( train_counts, train_arabic, open(outstem+".arabic.train.fpg",'w'))

    write_as_seg_pos(test_counts, test_hebrew, open(outstem+".hebrew.test.seg_pos",'w'))
    write_as_seg_pos(test_counts, test_arabic, open(outstem+".arabic.test.seg_pos",'w'))
    write_as_seg_pos(train_counts, train_hebrew, open(outstem+".hebrew.train.seg_pos",'w'))
    write_as_seg_pos(train_counts, train_arabic, open(outstem+".arabic.train.seg_pos",'w'))

if __name__ == '__main__':
    main()
